#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_appliance_traffic_shaping_custom_performance_classes
short_description: Resource module for networks _appliance _trafficshaping _customperformanceclasses
description:
  - Manage operation create of the resource networks _appliance _trafficshaping _customperformanceclasses.
  - Add a custom performance class for an MX network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  maxJitter:
    description: Maximum jitter in milliseconds.
    type: int
  maxLatency:
    description: Maximum latency in milliseconds.
    type: int
  maxLossPercentage:
    description: Maximum percentage of packet loss.
    type: int
  name:
    description: Name of the custom performance class.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for appliance createNetworkApplianceTrafficShapingCustomPerformanceClass
    description: Complete reference of the createNetworkApplianceTrafficShapingCustomPerformanceClass
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-network-appliance-traffic-shaping-custom-performance-class
notes:
  - SDK Method used are
    appliance.Appliance.create_network_appliance_traffic_shaping_custom_performance_class,
  - Paths used are
    post /networks/{networkId}/appliance/trafficShaping/customPerformanceClasses,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_appliance_traffic_shaping_custom_performance_classes:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    maxJitter: 100
    maxLatency: 100
    maxLossPercentage: 5
    name: myCustomPerformanceClass
    networkId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "name": "string",
      "customPerformanceClassId": "string",
      "maxLatency": 0,
      "maxJitter": 0,
      "maxLossPercentage": 0
    }
"""



#!/usr/bin/python
#
# Copyright 2018 www.privaz.io Valletech AB
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Make coding more python3-ish
from __future__ import annotations

DOCUMENTATION = r"""
module: one_host

short_description: Manages OpenNebula Hosts


requirements:
  - pyone

description:
  - Manages OpenNebula Hosts.
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none

options:
  name:
    description:
      - Hostname of the machine to manage.
    required: true
    type: str
  state:
    description:
      - Takes the host to the desired lifecycle state.
      - If V(absent) the host is deleted from the cluster.
      - If V(present) the host is created in the cluster (includes V(enabled), V(disabled) and V(offline) states).
      - If V(enabled) the host is fully operational.
      - V(disabled), for example to perform maintenance operations.
      - V(offline), host is totally offline.
    choices:
      - absent
      - present
      - enabled
      - disabled
      - offline
    default: present
    type: str
  im_mad_name:
    description:
      - The name of the information manager, this values are taken from the oned.conf with the tag name IM_MAD (name).
    default: kvm
    type: str
  vmm_mad_name:
    description:
      - The name of the virtual machine manager mad name, this values are taken from the oned.conf with the tag name VM_MAD
        (name).
    default: kvm
    type: str
  cluster_id:
    description:
      - The cluster ID.
    default: 0
    type: int
  cluster_name:
    description:
      - The cluster specified by name.
    type: str
  labels:
    description:
      - The labels for this host.
    type: list
    elements: str
  template:
    description:
      - The template or attribute changes to merge into the host template.
    aliases:
      - attributes
    type: dict

extends_documentation_fragment:
  - community.general._opennebula
  - community.general._attributes

author:
  - Rafael del Valle (@rvalle)
"""

EXAMPLES = r"""
- name: Create a new host in OpenNebula
  community.general.one_host:
    name: host1
    cluster_id: 1
    api_url: http://127.0.0.1:2633/RPC2

- name: Create a host and adjust its template
  community.general.one_host:
    name: host2
    cluster_name: default
    template:
      LABELS:
        - gold
        - ssd
      RESERVED_CPU: -100
"""

# TODO: pending setting guidelines on returned values
RETURN = r"""
"""

# TODO: Documentation on valid state transitions is required to properly implement all valid cases
# TODO: To be coherent with CLI this module should also provide "flush" functionality

from ansible_collections.community.general.plugins.module_utils._opennebula import OpenNebulaModule

try:
    from pyone import HOST_STATES, HOST_STATUS
except ImportError:
    pass  # handled at module utils


# Pseudo definitions...

HOST_ABSENT = -99  # the host is absent (special case defined by this module)


class HostModule(OpenNebulaModule):
    def __init__(self):
        argument_spec = dict(
            name=dict(type="str", required=True),
            state=dict(choices=["present", "absent", "enabled", "disabled", "offline"], default="present"),
            im_mad_name=dict(type="str", default="kvm"),
            vmm_mad_name=dict(type="str", default="kvm"),
            cluster_id=dict(type="int", default=0),
            cluster_name=dict(type="str"),
            labels=dict(type="list", elements="str"),
            template=dict(type="dict", aliases=["attributes"]),
        )

        mutually_exclusive = [["cluster_id", "cluster_name"]]

        OpenNebulaModule.__init__(self, argument_spec, mutually_exclusive=mutually_exclusive)

    def allocate_host(self):
        """
        Creates a host entry in OpenNebula
        self.one.host.allocate returns ID of a host
        Returns: True on success, fails otherwise.

        """
        try:
            self.one.host.allocate(
                self.get_parameter("name"),
                self.get_parameter("vmm_mad_name"),
                self.get_parameter("im_mad_name"),
                self.get_parameter("cluster_id"),
            )
            self.result["changed"] = True
        except Exception as e:
            self.fail(msg=f"Could not allocate host, ERROR: {e}")

        return True

    def wait_for_host_state(self, host, target_states):
        """
        Utility method that waits for a host state.
        Args:
            host:
            target_states:

        """
        return self.wait_for_state(
            "host",
            lambda: self.one.host.info(host.ID).STATE,
            lambda s: HOST_STATES(s).name,
            target_states,
            invalid_states=[HOST_STATES.ERROR, HOST_STATES.MONITORING_ERROR],
        )

    def run(self, one, module, result):
        # Get the list of hosts
        host_name = self.get_parameter("name")
        host = self.get_host_by_name(host_name)

        # manage host state
        desired_state = self.get_parameter("state")
        if bool(host):
            current_state = host.STATE
            current_state_name = HOST_STATES(host.STATE).name
        else:
            current_state = HOST_ABSENT
            current_state_name = "ABSENT"

        # apply properties
        if desired_state == "present":
            if current_state == HOST_ABSENT:
                self.allocate_host()
                host = self.get_host_by_name(host_name)
                self.wait_for_host_state(host, [HOST_STATES.MONITORED])
            elif current_state in [HOST_STATES.ERROR, HOST_STATES.MONITORING_ERROR]:
                self.fail(msg=f"invalid host state {current_state_name}")

        elif desired_state == "enabled":
            if current_state == HOST_ABSENT:
                self.allocate_host()
                host = self.get_host_by_name(host_name)
                self.wait_for_host_state(host, [HOST_STATES.MONITORED])
            elif current_state in [HOST_STATES.DISABLED, HOST_STATES.OFFLINE]:
                if one.host.status(host.ID, HOST_STATUS.ENABLED):
                    self.wait_for_host_state(host, [HOST_STATES.MONITORED])
                    result["changed"] = True
                else:
                    self.fail(msg="could not enable host")
            elif current_state in [HOST_STATES.MONITORED]:
                pass
            else:
                self.fail(msg=f"unknown host state {current_state_name}, cowardly refusing to change state to enable")

        elif desired_state == "disabled":
            if current_state == HOST_ABSENT:
                self.fail(msg="absent host cannot be put in disabled state")
            elif current_state in [HOST_STATES.MONITORED, HOST_STATES.OFFLINE]:
                # returns host ID integer
                try:
                    one.host.status(host.ID, HOST_STATUS.DISABLED)
                    result["changed"] = True
                except Exception as e:
                    self.fail(msg=f"Could not disable host, ERROR: {e}")
                self.wait_for_host_state(host, [HOST_STATES.DISABLED])
            elif current_state in [HOST_STATES.DISABLED]:
                pass
            else:
                self.fail(msg=f"unknown host state {current_state_name}, cowardly refusing to change state to disable")

        elif desired_state == "offline":
            if current_state == HOST_ABSENT:
                self.fail(msg="absent host cannot be placed in offline state")
            elif current_state in [HOST_STATES.MONITORED, HOST_STATES.DISABLED]:
                # returns host ID integer
                try:
                    one.host.status(host.ID, HOST_STATUS.OFFLINE)
                    result["changed"] = True
                except Exception as e:
                    self.fail(msg=f"Could not set host offline, ERROR: {e}")
                self.wait_for_host_state(host, [HOST_STATES.OFFLINE])
            elif current_state in [HOST_STATES.OFFLINE]:
                pass
            else:
                self.fail(msg=f"unknown host state {current_state_name}, cowardly refusing to change state to offline")

        elif desired_state == "absent":
            if current_state != HOST_ABSENT:
                # returns host ID integer
                try:
                    one.host.delete(host.ID)
                    result["changed"] = True
                except Exception as e:
                    self.fail(msg=f"Could not delete host from cluster, ERROR: {e}")

        # if we reach this point we can assume that the host was taken to the desired state

        if desired_state != "absent":
            # manipulate or modify the template
            desired_template_changes = self.get_parameter("template")

            if desired_template_changes is None:
                desired_template_changes = dict()

            # complete the template with specific ansible parameters
            if self.is_parameter("labels"):
                desired_template_changes["LABELS"] = self.get_parameter("labels")

            if self.requires_template_update(host.TEMPLATE, desired_template_changes):
                # setup the root element so that pyone will generate XML instead of attribute vector
                desired_template_changes = {"TEMPLATE": desired_template_changes}
                # merge the template, returns host ID integer
                try:
                    one.host.update(host.ID, desired_template_changes, 1)
                    result["changed"] = True
                except Exception as e:
                    self.fail(msg=f"Failed to update the host template, ERROR: {e}")

            # the cluster
            if self.get_parameter("cluster_id") != host.CLUSTER_ID:
                # returns cluster id in int
                try:
                    one.cluster.addhost(self.get_parameter("cluster_id"), host.ID)
                    result["changed"] = True
                except Exception as e:
                    self.fail(msg=f"Failed to update the host cluster, ERROR: {e}")

        # return
        self.exit()


def main():
    HostModule().run_module()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2018, Johannes Brunswicker <johannes.brunswicker@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: utm_proxy_location_info

author:
  - Johannes Brunswicker (@MatrixCrawler)

short_description: Create, update or destroy reverse_proxy location entry in Sophos UTM

description:
  - Create, update or destroy a reverse_proxy location entry in SOPHOS UTM.
  - This module needs to have the REST Ability of the UTM to be activated.
attributes:
  check_mode:
    version_added: 3.3.0
        # This was backported to 2.5.4 and 1.3.11 as well, since this was a bugfix

options:
  name:
    type: str
    description:
      - The name of the object that identifies the entry.
    required: true

extends_documentation_fragment:
  - community.general._utm
  - community.general._attributes
  - community.general._attributes.info_module
"""

EXAMPLES = r"""
- name: Remove UTM proxy_location
  community.general.utm_proxy_location_info:
    utm_host: sophos.host.name
    utm_token: abcdefghijklmno1234
    name: TestLocationEntry
"""

RETURN = r"""
result:
  description: The utm object that was created.
  returned: success
  type: complex
  contains:
    _ref:
      description: The reference name of the object.
      type: str
    _locked:
      description: Whether or not the object is currently locked.
      type: bool
    _type:
      description: The type of the object.
      type: str
    name:
      description: The name of the object.
      type: str
    access_control:
      description: Whether to use access control state.
      type: str
    allowed_networks:
      description: List of allowed network reference names.
      type: list
    auth_profile:
      description: The auth profile reference name.
      type: str
    backend:
      description: The backend reference name.
      type: str
    be_path:
      description: The backend path.
      type: str
    comment:
      description: The comment string.
      type: str
    denied_networks:
      description: The list of the denied network names.
      type: list
    hot_standby:
      description: Use hot standby.
      type: bool
    path:
      description: Path name.
      type: str
    status:
      description: Whether the object is active or not.
      type: bool
    stickysession_id:
      description: The identifier of the stickysession.
      type: str
    stickysession_status:
      description: Whether to use stickysession or not.
      type: bool
    websocket_passthrough:
      description: Whether websocket passthrough is used or not.
      type: bool
"""

from ansible_collections.community.general.plugins.module_utils._utm_utils import UTM, UTMModule


def main():
    endpoint = "reverse_proxy/location"
    key_to_check_for_changes = []
    module = UTMModule(
        argument_spec=dict(
            name=dict(type="str", required=True),
        ),
        supports_check_mode=True,
    )
    try:
        UTM(module, endpoint, key_to_check_for_changes, info_only=True).execute()
    except Exception as e:
        module.fail_json(msg=f"{e}")


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_summary_top_networks_by_status_info
short_description: Information module for organizations _summary _top _networks _bystatus
description:
  - Information module for Organizations Summary Top Networks Bystatus Info.
  - Get all organizations _summary _top _networks _bystatus.
  - >
    List the client and status overview information for the networks in an organization.
    Usage is measured in kilobytes and from the last seven days.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - Information module for Organizations Summary Top Networks Bystatus Info.
      - OrganizationId path parameter. Organization ID.
    type: str
  networkTag:
    description:
      - Information module for Organizations Summary Top Networks Bystatus Info.
      - NetworkTag query parameter. Match result to an exact network tag.
    type: str
  deviceTag:
    description:
      - Information module for Organizations Summary Top Networks Bystatus Info.
      - DeviceTag query parameter. Match result to an exact device tag.
    type: str
  quantity:
    description:
      - Information module for Organizations Summary Top Networks Bystatus Info.
      - Quantity query parameter. Set number of desired results to return. Default
        is 10. Maximum is 50.
    type: int
  ssidName:
    description:
      - Information module for Organizations Summary Top Networks Bystatus Info.
      - SsidName query parameter. Filter results by ssid name.
    type: str
  usageUplink:
    description:
      - Information module for Organizations Summary Top Networks Bystatus Info.
      - UsageUplink query parameter. Filter results by usage uplink.
    type: str
  perPage:
    description:
      - Information module for Organizations Summary Top Networks Bystatus Info.
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 5000.
    type: int
  startingAfter:
    description:
      - Information module for Organizations Summary Top Networks Bystatus Info.
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - Information module for Organizations Summary Top Networks Bystatus Info.
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationSummaryTopNetworksByStatus
    description: Complete reference of the getOrganizationSummaryTopNetworksByStatus
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-summary-top-networks-by-status
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_summary_top_networks_by_status,
  - Paths used are
    get /organizations/{organizationId}/summary/top/networks/byStatus,
"""

EXAMPLES = r"""
- name: Get all organizations _summary _top _networks _bystatus
  cisco.meraki.organizations_summary_top_networks_by_status_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    networkTag: string
    deviceTag: string
    quantity: 0
    ssidName: string
    usageUplink: string
    perPage: 0
    startingAfter: string
    endingBefore: string
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "networkId": "string",
        "name": "string",
        "url": "string",
        "tags": [
          "string"
        ],
        "clients": {
          "counts": {
            "total": 0
          },
          "usage": {
            "upstream": 0,
            "downstream": 0
          }
        },
        "statuses": {
          "overall": "string",
          "byProductType": [
            {
              "productType": "string",
              "counts": {
                "online": 0,
                "offline": 0,
                "alerting": 0,
                "dormant": 0
              }
            }
          ]
        },
        "devices": {
          "byProductType": [
            {
              "productType": "string",
              "url": "string"
            }
          ]
        },
        "productTypes": [
          "string"
        ]
      }
    ]
"""



#!/usr/bin/python
# Copyright (c) 2020, Adam Vaughan (@adamvaughan) avaughan@pagerduty.com
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: pagerduty_change
short_description: Track a code or infrastructure change as a PagerDuty change event
version_added: 1.3.0
description:
  - This module lets you create a PagerDuty change event each time the module is run.
  - This is not an idempotent action and a new change event is created each time it is run.
author:
  - Adam Vaughan (@adamvaughan)
requirements:
  - PagerDuty integration key
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
    details:
      - Check mode simply does nothing except returning RV(ignore:changed=true) in case the O(url) seems to be correct.
  diff_mode:
    support: none
options:
  integration_key:
    description:
      - The integration key that identifies the service the change was made to. This can be found by adding an integration
        to a service in PagerDuty.
    required: true
    type: str
  summary:
    description:
      - A short description of the change that occurred.
    required: true
    type: str
  source:
    description:
      - The source of the change event.
    default: Ansible
    type: str
  user:
    description:
      - The name of the user or process that triggered this deployment.
    type: str
  repo:
    description:
      - The URL of the project repository.
    type: str
  revision:
    description:
      - An identifier of the revision being deployed, typically a number or SHA from a version control system.
    type: str
  environment:
    description:
      - The environment name, typically V(production), V(staging), and so on.
    type: str
  link_url:
    description:
      - A URL where more information about the deployment can be obtained.
    type: str
  link_text:
    description:
      - Descriptive text for a URL where more information about the deployment can be obtained.
    type: str
  url:
    description:
      - URL to submit the change event to.
    default: https://events.pagerduty.com/v2/change/enqueue
    type: str
  validate_certs:
    description:
      - If V(false), SSL certificates for the target URL are not validated. This should only be used on personally controlled
        sites using self-signed certificates.
    default: true
    type: bool
"""

EXAMPLES = r"""
- name: Track the deployment as a PagerDuty change event
  community.general.pagerduty_change:
    integration_key: abc123abc123abc123abc123abc123ab
    summary: The application was deployed

- name: Track the deployment as a PagerDuty change event with more details
  community.general.pagerduty_change:
    integration_key: abc123abc123abc123abc123abc123ab
    summary: The application was deployed
    source: Ansible Deploy
    user: ansible
    repo: github.com/ansible/ansible
    revision: '4.2'
    environment: production
    link_url: https://github.com/ansible-collections/community.general/pull/1269
    link_text: View changes on GitHub
"""

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url

from ansible_collections.community.general.plugins.module_utils._datetime import (
    now,
)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            integration_key=dict(required=True, type="str", no_log=True),
            summary=dict(required=True, type="str"),
            source=dict(default="Ansible", type="str"),
            user=dict(type="str"),
            repo=dict(type="str"),
            revision=dict(type="str"),
            environment=dict(type="str"),
            link_url=dict(type="str"),
            link_text=dict(type="str"),
            url=dict(default="https://events.pagerduty.com/v2/change/enqueue", type="str"),
            validate_certs=dict(default=True, type="bool"),
        ),
        supports_check_mode=True,
    )

    # API documented at https://developer.pagerduty.com/docs/events-api-v2/send-change-events/

    url = module.params["url"]
    headers = {"Content-Type": "application/json"}

    if module.check_mode:
        _response, info = fetch_url(module, url, headers=headers, method="POST")

        if info["status"] == 400:
            module.exit_json(changed=True)
        else:
            module.fail_json(
                msg=f"Checking the PagerDuty change event API returned an unexpected response: {info['status']}"
            )

    custom_details = {}

    if module.params["user"]:
        custom_details["user"] = module.params["user"]

    if module.params["repo"]:
        custom_details["repo"] = module.params["repo"]

    if module.params["revision"]:
        custom_details["revision"] = module.params["revision"]

    if module.params["environment"]:
        custom_details["environment"] = module.params["environment"]

    timestamp = now().strftime("%Y-%m-%dT%H:%M:%S.%fZ")

    payload = {
        "summary": module.params["summary"],
        "source": module.params["source"],
        "timestamp": timestamp,
        "custom_details": custom_details,
    }

    event = {"routing_key": module.params["integration_key"], "payload": payload}

    if module.params["link_url"]:
        link = {"href": module.params["link_url"]}

        if module.params["link_text"]:
            link["text"] = module.params["link_text"]

        event["links"] = [link]

    _response, info = fetch_url(module, url, data=module.jsonify(event), headers=headers, method="POST")

    if info["status"] == 202:
        module.exit_json(changed=True)
    else:
        module.fail_json(msg=f"Creating PagerDuty change event failed with {info['status']}")


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2015, Matt Makai <matthew.makai@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: twilio
short_description: Sends a text message to a mobile phone through Twilio
description:
  - Sends a text message to a phone number through the Twilio messaging API.
notes:
  - This module is non-idempotent because it sends an email through the external API. It is idempotent only in the case that
    the module fails.
  - Like the other notification modules, this one requires an external dependency to work. In this case, you need a Twilio
    account with a purchased or verified phone number to send the text message.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  account_sid:
    type: str
    description:
      - User's Twilio account token found on the account page.
    required: true
  auth_token:
    type: str
    description:
      - User's Twilio authentication token.
    required: true
  msg:
    type: str
    description:
      - The body of the text message.
    required: true
  to_numbers:
    type: list
    elements: str
    description:
      - One or more phone numbers to send the text message to, format C(+15551112222).
    required: true
    aliases: [to_number]
  from_number:
    type: str
    description:
      - The Twilio number to send the text message from, format C(+15551112222).
    required: true
  media_url:
    type: str
    description:
      - A URL with a picture, video or sound clip to send with an MMS (multimedia message) instead of a plain SMS.

author: "Matt Makai (@makaimc)"
"""

EXAMPLES = r"""
# send an SMS about the build status to (555) 303 5681
# note: replace account_sid and auth_token values with your credentials
# and you have to have the 'from_number' on your Twilio account
- name: Send a text message to a mobile phone through Twilio
  community.general.twilio:
    msg: All servers with webserver role are now configured.
    account_sid: ACXXXXXXXXXXXXXXXXX
    auth_token: ACXXXXXXXXXXXXXXXXX
    from_number: "+15552014545"
    to_number: "+15553035681"
  delegate_to: localhost

# send an SMS to multiple phone numbers about the deployment
# note: replace account_sid and auth_token values with your credentials
# and you have to have the 'from_number' on your Twilio account
- name: Send a text message to a mobile phone through Twilio
  community.general.twilio:
    msg: This server configuration is now complete.
    account_sid: ACXXXXXXXXXXXXXXXXX
    auth_token: ACXXXXXXXXXXXXXXXXX
    from_number: "+15553258899"
    to_numbers:
      - "+15551113232"
      - "+12025551235"
      - "+19735559010"
  delegate_to: localhost

# send an MMS to a single recipient with an update on the deployment
# and an image of the results
# note: replace account_sid and auth_token values with your credentials
# and you have to have the 'from_number' on your Twilio account
- name: Send a text message to a mobile phone through Twilio
  community.general.twilio:
    msg: Deployment complete!
    account_sid: ACXXXXXXXXXXXXXXXXX
    auth_token: ACXXXXXXXXXXXXXXXXX
    from_number: "+15552014545"
    to_number: "+15553035681"
    media_url: https://demo.twilio.com/logo.png
  delegate_to: localhost
"""

# =======================================
# twilio module support methods
#
from urllib.parse import urlencode

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url


def post_twilio_api(module, account_sid, auth_token, msg, from_number, to_number, media_url=None):
    URI = f"https://api.twilio.com/2010-04-01/Accounts/{account_sid}/Messages.json"
    AGENT = "Ansible"

    data = {"From": from_number, "To": to_number, "Body": msg}
    if media_url:
        data["MediaUrl"] = media_url
    encoded_data = urlencode(data)

    headers = {
        "User-Agent": AGENT,
        "Content-type": "application/x-www-form-urlencoded",
        "Accept": "application/json",
    }

    # Hack module params to have the Basic auth params that fetch_url expects
    module.params["url_username"] = account_sid.replace("\n", "")
    module.params["url_password"] = auth_token.replace("\n", "")

    return fetch_url(module, URI, data=encoded_data, headers=headers)


# =======================================
# Main
#


def main():
    module = AnsibleModule(
        argument_spec=dict(
            account_sid=dict(required=True),
            auth_token=dict(required=True, no_log=True),
            msg=dict(required=True),
            from_number=dict(required=True),
            to_numbers=dict(required=True, aliases=["to_number"], type="list", elements="str"),
            media_url=dict(),
        ),
        supports_check_mode=True,
    )

    account_sid = module.params["account_sid"]
    auth_token = module.params["auth_token"]
    msg = module.params["msg"]
    from_number = module.params["from_number"]
    to_numbers = module.params["to_numbers"]
    media_url = module.params["media_url"]

    for number in to_numbers:
        r, info = post_twilio_api(module, account_sid, auth_token, msg, from_number, number, media_url)
        if info["status"] not in [200, 201]:
            body_message = "unknown error"
            if "body" in info:
                body = module.from_json(info["body"])
                body_message = body["message"]
            module.fail_json(msg=f"unable to send message to {number}: {body_message}")

    module.exit_json(msg=msg, changed=False)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Scaleway Security Group management module
#
# Copyright (C) 2018 Antoine Barbare (antoinebarbare@gmail.com).
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: scaleway_security_group
short_description: Scaleway Security Group management module
author: Antoine Barbare (@abarbare)
description:
  - This module manages Security Group on Scaleway account U(https://developer.scaleway.com).
extends_documentation_fragment:
  - community.general._scaleway
  - community.general._attributes
  - community.general._scaleway.actiongroup_scaleway

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
  action_group:
    version_added: 11.3.0

options:
  state:
    description:
      - Indicate desired state of the Security Group.
    type: str
    choices: [absent, present]
    default: present

  organization:
    type: str
    description:
      - Organization identifier.
      - Exactly one of O(project) and O(organization) must be specified.

  region:
    description:
      - Scaleway region to use (for example V(par1)).
    type: str
    required: true
    choices:
      - ams1
      - EMEA-NL-EVS
      - ams2
      - ams3
      - par1
      - EMEA-FR-PAR1
      - par2
      - EMEA-FR-PAR2
      - par3
      - waw1
      - EMEA-PL-WAW1
      - waw2
      - waw3

  name:
    description:
      - Name of the Security Group.
    type: str
    required: true

  description:
    description:
      - Description of the Security Group.
    type: str

  stateful:
    description:
      - Create a stateful security group which allows established connections in and out.
    type: bool
    required: true

  inbound_default_policy:
    description:
      - Default policy for incoming traffic.
    type: str
    choices: [accept, drop]

  outbound_default_policy:
    description:
      - Default policy for outcoming traffic.
    type: str
    choices: [accept, drop]

  organization_default:
    description:
      - Create security group to be the default one.
    type: bool

  project:
    type: str
    description:
      - Project identifier.
      - Exactly one of O(project) and O(organization) must be specified.
    version_added: 12.3.0
"""

EXAMPLES = r"""
- name: Create a Security Group using a project ID
  community.general.scaleway_security_group:
    state: present
    region: par1
    name: security_group
    description: "my security group description"
    stateful: false
    inbound_default_policy: accept
    outbound_default_policy: accept
    organization_default: false
    project: 951df375-e094-4d26-97c1-ba548eeb9c42
  register: security_group_creation_task

- name: Create a Security Group in the default project using an organization ID (deprecated)
  community.general.scaleway_security_group:
    state: present
    region: par1
    name: security_group
    description: "my security group description"
    organization: 43a3b6c8-916f-477b-b7ec-ff1898f5fdd9
    stateful: false
    inbound_default_policy: accept
    outbound_default_policy: accept
    organization_default: false
  register: security_group_creation_task
"""

RETURN = r"""
data:
  description: This is only present when O(state=present).
  returned: when O(state=present)
  type: dict
  sample:
    {
      "scaleway_security_group": {
        "description": "my security group description",
        "enable_default_security": true,
        "id": "0168fb1f-cc46-4f69-b4be-c95d2a19bcae",
        "inbound_default_policy": "accept",
        "name": "security_group",
        "organization": "43a3b6c8-916f-477b-b7ec-ff1898f5fdd9",
        "organization_default": false,
        "outbound_default_policy": "accept",
        "servers": [],
        "stateful": false
      }
    }
"""

from uuid import uuid4

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._scaleway import (
    SCALEWAY_LOCATION,
    Scaleway,
    scaleway_argument_spec,
)


def payload_from_security_group(security_group):
    return {k: v for k, v in security_group.items() if k != "id" and v is not None}


def present_strategy(api, security_group):
    ret = {"changed": False}

    response = api.get("security_groups")
    if not response.ok:
        api.module.fail_json(
            msg=f'Error getting security groups "{response.info["msg"]}": "{response.json["message"]}" ({response.json})'
        )

    security_group_lookup = {sg["name"]: sg for sg in response.json["security_groups"]}

    if security_group["name"] not in security_group_lookup.keys():
        ret["changed"] = True
        if api.module.check_mode:
            # Help user when check mode is enabled by defining id key
            ret["scaleway_security_group"] = {"id": str(uuid4())}
            return ret

        # Create Security Group
        response = api.post("/security_groups", data=payload_from_security_group(security_group))

        if not response.ok:
            msg = f'Error during security group creation: "{response.info["msg"]}": "{response.json["message"]}" ({response.json})'
            api.module.fail_json(msg=msg)
        ret["scaleway_security_group"] = response.json["security_group"]

    else:
        ret["scaleway_security_group"] = security_group_lookup[security_group["name"]]

    return ret


def absent_strategy(api, security_group):
    response = api.get("security_groups")
    ret = {"changed": False}

    if not response.ok:
        api.module.fail_json(
            msg=f'Error getting security groups "{response.info["msg"]}": "{response.json["message"]}" ({response.json})'
        )

    security_group_lookup = {sg["name"]: sg for sg in response.json["security_groups"]}
    if security_group["name"] not in security_group_lookup.keys():
        return ret

    ret["changed"] = True
    if api.module.check_mode:
        return ret

    response = api.delete(f"/security_groups/{security_group_lookup[security_group['name']]['id']}")
    if not response.ok:
        api.module.fail_json(
            msg=f'Error deleting security group "{response.info["msg"]}": "{response.json["message"]}" ({response.json})'
        )

    return ret


def core(module):
    security_group = {
        "organization": module.params["organization"],
        "name": module.params["name"],
        "description": module.params["description"],
        "stateful": module.params["stateful"],
        "inbound_default_policy": module.params["inbound_default_policy"],
        "outbound_default_policy": module.params["outbound_default_policy"],
        "organization_default": module.params["organization_default"],
        "project": module.params["project"],
    }

    region = module.params["region"]
    module.params["api_url"] = SCALEWAY_LOCATION[region]["api_endpoint"]

    api = Scaleway(module=module)
    if module.params["state"] == "present":
        summary = present_strategy(api=api, security_group=security_group)
    else:
        summary = absent_strategy(api=api, security_group=security_group)
    module.exit_json(**summary)


def main():
    argument_spec = scaleway_argument_spec()
    argument_spec.update(
        dict(
            state=dict(type="str", default="present", choices=["absent", "present"]),
            organization=dict(type="str"),
            name=dict(type="str", required=True),
            description=dict(type="str"),
            region=dict(type="str", required=True, choices=list(SCALEWAY_LOCATION.keys())),
            stateful=dict(type="bool", required=True),
            inbound_default_policy=dict(type="str", choices=["accept", "drop"]),
            outbound_default_policy=dict(type="str", choices=["accept", "drop"]),
            organization_default=dict(type="bool"),
            project=dict(type="str"),
        )
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[["stateful", True, ["inbound_default_policy", "outbound_default_policy"]]],
        mutually_exclusive=[
            ("organization", "project"),
        ],
        required_one_of=[
            ("organization", "project"),
        ],
    )

    core(module)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2019, Sandeep Kasargod (sandeep@vexata.com)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: vexata_volume
short_description: Manage volumes on Vexata VX100 storage arrays
description:
  - Create, deletes or extend volumes on a Vexata VX100 array.
author:
  - Sandeep Kasargod (@vexata)
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    description:
      - Volume name.
    required: true
    type: str
  state:
    description:
      - Creates/Modifies volume when present or removes when absent.
    default: present
    choices: [present, absent]
    type: str
  size:
    description:
      - Volume size in M, G, T units. M=2^20, G=2^30, T=2^40 bytes.
    type: str
extends_documentation_fragment:
  - community.general._vexata.vx100
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Create new 2 TiB volume named foo
  community.general.vexata_volume:
    name: foo
    size: 2T
    state: present
    array: vx100_ultra.test.com
    user: admin
    password: secret

- name: Expand volume named foo to 4 TiB
  community.general.vexata_volume:
    name: foo
    size: 4T
    state: present
    array: vx100_ultra.test.com
    user: admin
    password: secret

- name: Delete volume named foo
  community.general.vexata_volume:
    name: foo
    state: absent
    array: vx100_ultra.test.com
    user: admin
    password: secret
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._vexata import (
    argument_spec,
    get_array,
    required_together,
    size_to_MiB,
)


def get_volume(module, array):
    """Retrieve a named volume if it exists, None if absent."""
    name = module.params["name"]
    try:
        vols = array.list_volumes()
        vol = [v for v in vols if v["name"] == name]
        if len(vol) == 1:
            return vol[0]
        else:
            return None
    except Exception:
        module.fail_json(msg="Error while attempting to retrieve volumes.")


def validate_size(module, err_msg):
    size = module.params.get("size", False)
    if not size:
        module.fail_json(msg=err_msg)
    size = size_to_MiB(size)
    if size <= 0:
        module.fail_json(msg="Invalid volume size, must be <integer>[MGT].")
    return size


def create_volume(module, array):
    """ "Create a new volume."""
    changed = False
    size = validate_size(module, err_msg="Size is required to create volume.")
    if module.check_mode:
        module.exit_json(changed=changed)

    try:
        vol = array.create_volume(module.params["name"], "Ansible volume", size)
        if vol:
            module.log(msg=f"Created volume {vol['id']}")
            changed = True
        else:
            module.fail_json(msg="Volume create failed.")
    except Exception:
        pass
    module.exit_json(changed=changed)


def update_volume(module, array, volume):
    """Expand the volume size."""
    changed = False
    size = validate_size(module, err_msg="Size is required to update volume")
    prev_size = volume["volSize"]
    if size <= prev_size:
        module.log(msg="Volume expanded size needs to be larger than current size.")
    if module.check_mode:
        module.exit_json(changed=changed)

    try:
        vol = array.grow_volume(volume["name"], volume["description"], volume["id"], size)
        if vol:
            changed = True
    except Exception:
        pass

    module.exit_json(changed=changed)


def delete_volume(module, array, volume):
    changed = False
    vol_name = volume["name"]
    if module.check_mode:
        module.exit_json(changed=changed)

    try:
        ok = array.delete_volume(volume["id"])
        if ok:
            module.log(msg=f"Volume {vol_name} deleted.")
            changed = True
        else:
            raise Exception
    except Exception:
        pass
    module.exit_json(changed=changed)


def main():
    arg_spec = argument_spec()
    arg_spec.update(
        dict(
            name=dict(type="str", required=True),
            state=dict(default="present", choices=["present", "absent"]),
            size=dict(type="str"),
        )
    )

    module = AnsibleModule(arg_spec, supports_check_mode=True, required_together=required_together())

    state = module.params["state"]
    array = get_array(module)
    volume = get_volume(module, array)

    if state == "present":
        if not volume:
            create_volume(module, array)
        else:
            update_volume(module, array, volume)
    elif state == "absent" and volume:
        delete_volume(module, array, volume)
    else:
        module.exit_json(changed=False)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2018, Sebastian Schenzel <sebastian.schenzel@mailbox.org>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: utm_proxy_exception

author:
  - Sebastian Schenzel (@RickS-C137)

short_description: Create, update or destroy reverse_proxy exception entry in Sophos UTM

description:
  - Create, update or destroy a reverse_proxy exception entry in SOPHOS UTM.
  - This module needs to have the REST Ability of the UTM to be activated.
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none

options:
  name:
    description:
      - The name of the object that identifies the entry.
    required: true
    type: str
  op:
    description:
      - The operand to be used with the entries of the path parameter.
    default: 'AND'
    choices:
      - 'AND'
      - 'OR'
    type: str
  path:
    description:
      - The paths the exception in the reverse proxy is defined for.
    type: list
    elements: str
    default: []
  skip_custom_threats_filters:
    description:
      - A list of threats to be skipped.
    type: list
    elements: str
    default: []
  skip_threats_filter_categories:
    description:
      - Define which categories of threats are skipped.
    type: list
    elements: str
    default: []
  skipav:
    description:
      - Skip the Antivirus Scanning.
    default: false
    type: bool
  skipbadclients:
    description:
      - Block clients with bad reputation.
    default: false
    type: bool
  skipcookie:
    description:
      - Skip the Cookie Signing check.
    default: false
    type: bool
  skipform:
    description:
      - Enable form hardening.
    default: false
    type: bool
  skipform_missingtoken:
    description:
      - Enable form hardening with missing tokens.
    default: false
    type: bool
  skiphtmlrewrite:
    description:
      - Protection against SQL.
    default: false
    type: bool
  skiptft:
    description:
      - Enable true file type control.
    default: false
    type: bool
  skipurl:
    description:
      - Enable static URL hardening.
    default: false
    type: bool
  source:
    description:
      - Define which categories of threats are skipped.
    type: list
    elements: str
    default: []
  status:
    description:
      - Status of the exception rule set.
    default: true
    type: bool

extends_documentation_fragment:
  - community.general._utm
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Create UTM proxy_exception
  community.general.utm_proxy_exception:
    utm_host: sophos.host.name
    utm_token: abcdefghijklmno1234
    name: TestExceptionEntry
    backend: REF_OBJECT_STRING
    state: present

- name: Remove UTM proxy_exception
  community.general.utm_proxy_exception:
    utm_host: sophos.host.name
    utm_token: abcdefghijklmno1234
    name: TestExceptionEntry
    state: absent
"""

RETURN = r"""
result:
  description: The utm object that was created.
  returned: success
  type: complex
  contains:
    _ref:
      description: The reference name of the object.
      type: str
    _locked:
      description: Whether or not the object is currently locked.
      type: bool
    _type:
      description: The type of the object.
      type: str
    name:
      description: The name of the object.
      type: str
    comment:
      description: The optional comment string.
      type: str
    op:
      description: The operand to be used with the entries of the path parameter.
      type: str
    path:
      description: The paths the exception in the reverse proxy is defined for.
      type: list
    skip_custom_threats_filters:
      description: A list of threats to be skipped.
      type: list
    skip_threats_filter_categories:
      description: Define which categories of threats are skipped.
      type: list
    skipav:
      description: Skip the Antivirus Scanning.
      type: bool
    skipbadclients:
      description: Block clients with bad reputation.
      type: bool
    skipcookie:
      description: Skip the Cookie Signing check.
      type: bool
    skipform:
      description: Enable form hardening.
      type: bool
    skipform_missingtoken:
      description: Enable form hardening with missing tokens.
      type: bool
    skiphtmlrewrite:
      description: Protection against SQL.
      type: bool
    skiptft:
      description: Enable true file type control.
      type: bool
    skipurl:
      description: Enable static URL hardening.
      type: bool
    source:
      description: Define which categories of threats are skipped.
      type: list
"""

from ansible_collections.community.general.plugins.module_utils._utm_utils import UTM, UTMModule


def main():
    endpoint = "reverse_proxy/exception"
    key_to_check_for_changes = [
        "op",
        "path",
        "skip_custom_threats_filters",
        "skip_threats_filter_categories",
        "skipav",
        "comment",
        "skipbadclients",
        "skipcookie",
        "skipform",
        "status",
        "skipform_missingtoken",
        "skiphtmlrewrite",
        "skiptft",
        "skipurl",
        "source",
    ]
    module = UTMModule(
        argument_spec=dict(
            name=dict(type="str", required=True),
            op=dict(type="str", default="AND", choices=["AND", "OR"]),
            path=dict(type="list", elements="str", default=[]),
            skip_custom_threats_filters=dict(type="list", elements="str", default=[]),
            skip_threats_filter_categories=dict(type="list", elements="str", default=[]),
            skipav=dict(type="bool", default=False),
            skipbadclients=dict(type="bool", default=False),
            skipcookie=dict(type="bool", default=False),
            skipform=dict(type="bool", default=False),
            skipform_missingtoken=dict(type="bool", default=False),
            skiphtmlrewrite=dict(type="bool", default=False),
            skiptft=dict(type="bool", default=False),
            skipurl=dict(type="bool", default=False),
            source=dict(type="list", elements="str", default=[]),
            status=dict(type="bool", default=True),
        )
    )
    try:
        UTM(module, endpoint, key_to_check_for_changes).execute()
    except Exception as e:
        module.fail_json(msg=f"{e}")


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2021, Andreas Botzner <andreas at botzner dot com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: redis_data_info
short_description: Get value of key in Redis database
version_added: 3.7.0
description:
  - Get value of keys in Redis database.
author: "Andreas Botzner (@paginabianca)"
options:
  key:
    description:
      - Database key.
    type: str
    required: true

extends_documentation_fragment:
  - community.general._redis
  - community.general._attributes
  - community.general._attributes.info_module

seealso:
  - module: community.general.redis_data
  - module: community.general.redis_data_incr
  - module: community.general.redis_info
  - module: community.general.redis
"""

EXAMPLES = r"""
- name: Get key foo=bar from loalhost with no username
  community.general.redis_data_info:
    login_host: localhost
    login_password: supersecret
    key: foo

- name: Get key foo=bar on redishost with custom ca-cert file
  community.general.redis_data_info:
    login_host: redishost
    login_password: supersecret
    login_user: somuser
    validate_certs: true
    ssl_ca_certs: /path/to/ca/certs
    key: foo
"""

RETURN = r"""
exists:
  description: If they key exists in the database.
  returned: on success
  type: bool
value:
  description: Value key was set to.
  returned: if existing
  type: str
  sample: 'value_of_some_key'
msg:
  description: A short message.
  returned: always
  type: str
  sample: 'Got key: foo with value: bar'
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._redis import (
    RedisAnsible,
    fail_imports,
    redis_auth_argument_spec,
)


def main():
    redis_auth_args = redis_auth_argument_spec()
    module_args = dict(
        key=dict(type="str", required=True, no_log=False),
    )
    module_args.update(redis_auth_args)

    module = AnsibleModule(
        argument_spec=module_args,
        supports_check_mode=True,
    )
    fail_imports(module)

    redis = RedisAnsible(module)

    key = module.params["key"]
    result = {"changed": False}

    value = None
    try:
        value = redis.connection.get(key)
    except Exception as e:
        msg = f'Failed to get value of key "{key}" with exception: {e}'
        result["msg"] = msg
        module.fail_json(**result)

    if value is None:
        msg = f'Key "{key}" does not exist in database'
        result["exists"] = False
    else:
        msg = f'Got key "{key}"'
        result["value"] = value
        result["exists"] = True
    result["msg"] = msg
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2018, Stephan Schwarz <stearz@gmx.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: utm_proxy_auth_profile

author:
  - Stephan Schwarz (@stearz)

short_description: Create, update or destroy reverse_proxy auth_profile entry in Sophos UTM

description:
  - Create, update or destroy a reverse_proxy auth_profile entry in SOPHOS UTM.
  - This module needs to have the REST Ability of the UTM to be activated.
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none

options:
  name:
    type: str
    description:
      - The name of the object that identifies the entry.
    required: true
  aaa:
    type: list
    elements: str
    description:
      - List of references to utm_aaa objects (allowed users or groups).
    required: true
  basic_prompt:
    type: str
    description:
      - The message in the basic authentication prompt.
    required: true
  backend_mode:
    type: str
    description:
      - Specifies if the backend server needs authentication ([Basic|None]).
    default: None
    choices:
      - Basic
      - None
  backend_strip_basic_auth:
    description:
      - Should the login data be stripped when proxying the request to the backend host.
    type: bool
    default: true
  backend_user_prefix:
    type: str
    description:
      - Prefix string to prepend to the username for backend authentication.
    default: ""
  backend_user_suffix:
    type: str
    description:
      - Suffix string to append to the username for backend authentication.
    default: ""
  comment:
    type: str
    description:
      - Optional comment string.
    default: ""
  frontend_cookie:
    type: str
    description:
      - Frontend cookie name.
  frontend_cookie_secret:
    type: str
    description:
      - Frontend cookie secret.
  frontend_form:
    type: str
    description:
      - Frontend authentication form name.
  frontend_form_template:
    type: str
    description:
      - Frontend authentication form template.
    default: ""
  frontend_login:
    type: str
    description:
      - Frontend login name.
  frontend_logout:
    type: str
    description:
      - Frontend logout name.
  frontend_mode:
    type: str
    description:
      - Frontend authentication mode (Form|Basic).
    default: Basic
    choices:
      - Basic
      - Form
  frontend_realm:
    type: str
    description:
      - Frontend authentication realm.
  frontend_session_allow_persistency:
    description:
      - Allow session persistency.
    type: bool
    default: false
  frontend_session_lifetime:
    type: int
    description:
      - Session lifetime.
    required: true
  frontend_session_lifetime_limited:
    description:
      - Specifies if limitation of session lifetime is active.
    type: bool
    default: true
  frontend_session_lifetime_scope:
    type: str
    description:
      - Scope for frontend_session_lifetime (days|hours|minutes).
    default: hours
    choices:
      - days
      - hours
      - minutes
  frontend_session_timeout:
    type: int
    description:
      - Session timeout.
    required: true
  frontend_session_timeout_enabled:
    description:
      - Specifies if session timeout is active.
    type: bool
    default: true
  frontend_session_timeout_scope:
    type: str
    description:
      - Scope for frontend_session_timeout (days|hours|minutes).
    default: minutes
    choices:
      - days
      - hours
      - minutes
  logout_delegation_urls:
    type: list
    elements: str
    description:
      - List of logout URLs that logouts are delegated to.
    default: []
  logout_mode:
    type: str
    description:
      - Mode of logout (None|Delegation).
    default: None
    choices:
      - None
      - Delegation
  redirect_to_requested_url:
    description:
      - Should a redirect to the requested URL be made.
    type: bool
    default: false

extends_documentation_fragment:
  - community.general._utm
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Create UTM proxy_auth_profile
  community.general.utm_proxy_auth_profile:
    utm_host: sophos.host.name
    utm_token: abcdefghijklmno1234
    name: TestAuthProfileEntry
    aaa: [REF_OBJECT_STRING, REF_ANOTHEROBJECT_STRING]
    basic_prompt: "Authentication required: Please login"
    frontend_session_lifetime: 1
    frontend_session_timeout: 1
    state: present

- name: Remove UTM proxy_auth_profile
  community.general.utm_proxy_auth_profile:
    utm_host: sophos.host.name
    utm_token: abcdefghijklmno1234
    name: TestAuthProfileEntry
    state: absent

- name: Read UTM proxy_auth_profile
  community.general.utm_proxy_auth_profile:
    utm_host: sophos.host.name
    utm_token: abcdefghijklmno1234
    name: TestAuthProfileEntry
    state: info
"""

RETURN = r"""
result:
  description: The utm object that was created.
  returned: success
  type: complex
  contains:
    _ref:
      description: The reference name of the object.
      type: str
    _locked:
      description: Whether or not the object is currently locked.
      type: bool
    _type:
      description: The type of the object.
      type: str
    name:
      description: The name of the object.
      type: str
    aaa:
      description: List of references to utm_aaa objects (allowed users or groups).
      type: list
    basic_prompt:
      description: The message in the basic authentication prompt.
      type: str
    backend_mode:
      description: Specifies if the backend server needs authentication ([Basic|None]).
      type: str
    backend_strip_basic_auth:
      description: Should the login data be stripped when proxying the request to the backend host.
      type: bool
    backend_user_prefix:
      description: Prefix string to prepend to the username for backend authentication.
      type: str
    backend_user_suffix:
      description: Suffix string to append to the username for backend authentication.
      type: str
    comment:
      description: Optional comment string.
      type: str
    frontend_cookie:
      description: Frontend cookie name.
      type: str
    frontend_form:
      description: Frontend authentication form name.
      type: str
    frontend_form_template:
      description: Frontend authentication form template.
      type: str
    frontend_login:
      description: Frontend login name.
      type: str
    frontend_logout:
      description: Frontend logout name.
      type: str
    frontend_mode:
      description: Frontend authentication mode (Form|Basic).
      type: str
    frontend_realm:
      description: Frontend authentication realm.
      type: str
    frontend_session_allow_persistency:
      description: Allow session persistency.
      type: bool
    frontend_session_lifetime:
      description: Session lifetime.
      type: int
    frontend_session_lifetime_limited:
      description: Specifies if limitation of session lifetime is active.
      type: bool
    frontend_session_lifetime_scope:
      description: Scope for frontend_session_lifetime (days|hours|minutes).
      type: str
    frontend_session_timeout:
      description: Session timeout.
      type: int
    frontend_session_timeout_enabled:
      description: Specifies if session timeout is active.
      type: bool
    frontend_session_timeout_scope:
      description: Scope for frontend_session_timeout (days|hours|minutes).
      type: str
    logout_delegation_urls:
      description: List of logout URLs that logouts are delegated to.
      type: list
    logout_mode:
      description: Mode of logout (None|Delegation).
      type: str
    redirect_to_requested_url:
      description: Should a redirect to the requested URL be made.
      type: bool
"""

from ansible_collections.community.general.plugins.module_utils._utm_utils import UTM, UTMModule


def main():
    endpoint = "reverse_proxy/auth_profile"
    key_to_check_for_changes = [
        "aaa",
        "basic_prompt",
        "backend_mode",
        "backend_strip_basic_auth",
        "backend_user_prefix",
        "backend_user_suffix",
        "comment",
        "frontend_cookie",
        "frontend_cookie_secret",
        "frontend_form",
        "frontend_form_template",
        "frontend_login",
        "frontend_logout",
        "frontend_mode",
        "frontend_realm",
        "frontend_session_allow_persistency",
        "frontend_session_lifetime",
        "frontend_session_lifetime_limited",
        "frontend_session_lifetime_scope",
        "frontend_session_timeout",
        "frontend_session_timeout_enabled",
        "frontend_session_timeout_scope",
        "logout_delegation_urls",
        "logout_mode",
        "redirect_to_requested_url",
    ]

    module = UTMModule(
        argument_spec=dict(
            name=dict(type="str", required=True),
            aaa=dict(type="list", elements="str", required=True),
            basic_prompt=dict(type="str", required=True),
            backend_mode=dict(type="str", default="None", choices=["Basic", "None"]),
            backend_strip_basic_auth=dict(type="bool", default=True),
            backend_user_prefix=dict(type="str", default=""),
            backend_user_suffix=dict(type="str", default=""),
            comment=dict(type="str", default=""),
            frontend_cookie=dict(type="str"),
            frontend_cookie_secret=dict(type="str", no_log=True),
            frontend_form=dict(type="str"),
            frontend_form_template=dict(type="str", default=""),
            frontend_login=dict(type="str"),
            frontend_logout=dict(type="str"),
            frontend_mode=dict(type="str", default="Basic", choices=["Basic", "Form"]),
            frontend_realm=dict(type="str"),
            frontend_session_allow_persistency=dict(type="bool", default=False),
            frontend_session_lifetime=dict(type="int", required=True),
            frontend_session_lifetime_limited=dict(type="bool", default=True),
            frontend_session_lifetime_scope=dict(type="str", default="hours", choices=["days", "hours", "minutes"]),
            frontend_session_timeout=dict(type="int", required=True),
            frontend_session_timeout_enabled=dict(type="bool", default=True),
            frontend_session_timeout_scope=dict(type="str", default="minutes", choices=["days", "hours", "minutes"]),
            logout_delegation_urls=dict(type="list", elements="str", default=[]),
            logout_mode=dict(type="str", default="None", choices=["None", "Delegation"]),
            redirect_to_requested_url=dict(type="bool", default=False),
        )
    )
    try:
        UTM(module, endpoint, key_to_check_for_changes).execute()
    except Exception as e:
        module.fail_json(msg=f"{e}")


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2016-2017 Hewlett Packard Enterprise Development LP
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: oneview_ethernet_network_info
short_description: Retrieve the information about one or more of the OneView Ethernet Networks
description:
  - Retrieve the information about one or more of the Ethernet Networks from OneView.
requirements:
  - hpOneView >= 2.0.1
author:
  - Felipe Bulsoni (@fgbulsoni)
  - Thiago Miotto (@tmiotto)
  - Adriane Cardozo (@adriane-cardozo)
attributes:
  check_mode:
    version_added: 3.3.0
    # This was backported to 2.5.4 and 1.3.11 as well, since this was a bugfix
options:
  name:
    description:
      - Ethernet Network name.
    type: str
  options:
    description:
      - 'List with options to gather additional information about an Ethernet Network and related resources. Options allowed:
        V(associatedProfiles) and V(associatedUplinkGroups).'
    type: list
    elements: str
extends_documentation_fragment:
  - community.general._oneview
  - community.general._oneview.factsparams
  - community.general._attributes
  - community.general._attributes.info_module
"""

EXAMPLES = r"""
- name: Gather information about all Ethernet Networks
  community.general.oneview_ethernet_network_info:
    config: /etc/oneview/oneview_config.json
  delegate_to: localhost
  register: result

- name: Print fetched information about Ethernet Networks
  ansible.builtin.debug:
    msg: "{{ result.ethernet_networks }}"

- name: Gather paginated and filtered information about Ethernet Networks
  community.general.oneview_ethernet_network_info:
    config: /etc/oneview/oneview_config.json
    params:
      start: 1
      count: 3
      sort: 'name:descending'
      filter: 'purpose=General'
  delegate_to: localhost
  register: result

- name: Print fetched information about paginated and filtered list of Ethernet Networks
  ansible.builtin.debug:
    msg: "{{ result.ethernet_networks }}"

- name: Gather information about an Ethernet Network by name
  community.general.oneview_ethernet_network_info:
    config: /etc/oneview/oneview_config.json
    name: Ethernet network name
  delegate_to: localhost
  register: result

- name: Print fetched information about Ethernet Network found by name
  ansible.builtin.debug:
    msg: "{{ result.ethernet_networks }}"

- name: Gather information about an Ethernet Network by name with options
  community.general.oneview_ethernet_network_info:
    config: /etc/oneview/oneview_config.json
    name: eth1
    options:
      - associatedProfiles
      - associatedUplinkGroups
  delegate_to: localhost
  register: result

- name: Print fetched information about Ethernet Network Associated Profiles
  ansible.builtin.debug:
    msg: "{{ result.enet_associated_profiles }}"

- name: Print fetched information about Ethernet Network Associated Uplink Groups
  ansible.builtin.debug:
    msg: "{{ result.enet_associated_uplink_groups }}"
"""

RETURN = r"""
ethernet_networks:
  description: Has all the OneView information about the Ethernet Networks.
  returned: Always, but can be null.
  type: dict

enet_associated_profiles:
  description: Has all the OneView information about the profiles which are using the Ethernet network.
  returned: When requested, but can be null.
  type: dict

enet_associated_uplink_groups:
  description: Has all the OneView information about the uplink sets which are using the Ethernet network.
  returned: When requested, but can be null.
  type: dict
"""

from ansible_collections.community.general.plugins.module_utils._oneview import OneViewModuleBase


class EthernetNetworkInfoModule(OneViewModuleBase):
    argument_spec = dict(name=dict(type="str"), options=dict(type="list", elements="str"), params=dict(type="dict"))

    def __init__(self):
        super().__init__(
            additional_arg_spec=self.argument_spec,
            supports_check_mode=True,
        )

        self.resource_client = self.oneview_client.ethernet_networks

    def execute_module(self):
        info = {}
        if self.module.params["name"]:
            ethernet_networks = self.resource_client.get_by("name", self.module.params["name"])

            if self.module.params.get("options") and ethernet_networks:
                info = self.__gather_optional_info(ethernet_networks[0])
        else:
            ethernet_networks = self.resource_client.get_all(**self.facts_params)

        info["ethernet_networks"] = ethernet_networks

        return dict(changed=False, **info)

    def __gather_optional_info(self, ethernet_network):
        info = {}

        if self.options.get("associatedProfiles"):
            info["enet_associated_profiles"] = self.__get_associated_profiles(ethernet_network)
        if self.options.get("associatedUplinkGroups"):
            info["enet_associated_uplink_groups"] = self.__get_associated_uplink_groups(ethernet_network)

        return info

    def __get_associated_profiles(self, ethernet_network):
        associated_profiles = self.resource_client.get_associated_profiles(ethernet_network["uri"])
        return [self.oneview_client.server_profiles.get(x) for x in associated_profiles]

    def __get_associated_uplink_groups(self, ethernet_network):
        uplink_groups = self.resource_client.get_associated_uplink_groups(ethernet_network["uri"])
        return [self.oneview_client.uplink_sets.get(x) for x in uplink_groups]


def main():
    EthernetNetworkInfoModule().run()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright 2014 Peter Oliver <ansible@mavit.org.uk>
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: pkg5_publisher
author: "Peter Oliver (@mavit)"
short_description: Manages Solaris 11 Image Packaging System publishers
description:
  - IPS packages are the native packages in Solaris 11 and higher.
  - This module configures which publishers a client downloads IPS packages from.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  name:
    description:
      - The publisher's name.
    required: true
    aliases: [publisher]
    type: str
  state:
    description:
      - Whether to ensure that a publisher is present or absent.
    default: present
    choices: [present, absent]
    type: str
  sticky:
    description:
      - Packages installed from a sticky repository can only receive updates from that repository.
    type: bool
  enabled:
    description:
      - Is the repository enabled or disabled?
    type: bool
  origin:
    description:
      - A path or URL to the repository.
      - Multiple values may be provided.
    type: list
    elements: str
  mirror:
    description:
      - A path or URL to the repository mirror.
      - Multiple values may be provided.
    type: list
    elements: str
"""
EXAMPLES = r"""
- name: Fetch packages for the solaris publisher direct from Oracle
  community.general.pkg5_publisher:
    name: solaris
    sticky: true
    origin: https://pkg.oracle.com/solaris/support/

- name: Configure a publisher for locally-produced packages
  community.general.pkg5_publisher:
    name: site
    origin: 'https://pkg.example.com/site/'
"""

from ansible.module_utils.basic import AnsibleModule


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(required=True, aliases=["publisher"]),
            state=dict(default="present", choices=["present", "absent"]),
            sticky=dict(type="bool"),
            enabled=dict(type="bool"),
            # search_after=dict(),
            # search_before=dict(),
            origin=dict(type="list", elements="str"),
            mirror=dict(type="list", elements="str"),
        )
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    for option in ["origin", "mirror"]:
        if module.params[option] == [""]:
            module.params[option] = []

    if module.params["state"] == "present":
        modify_publisher(module, module.params)
    else:
        unset_publisher(module, module.params["name"])


def modify_publisher(module, params):
    name = params["name"]
    existing = get_publishers(module)

    if name in existing:
        for option in ["origin", "mirror", "sticky", "enabled"]:
            if params[option] is not None:
                if params[option] != existing[name][option]:
                    return set_publisher(module, params)
    else:
        return set_publisher(module, params)

    module.exit_json()


def set_publisher(module, params):
    name = params["name"]
    args = []

    if params["origin"] is not None:
        args.append("--remove-origin=*")
        args.extend([f"--add-origin={u}" for u in params["origin"]])
    if params["mirror"] is not None:
        args.append("--remove-mirror=*")
        args.extend([f"--add-mirror={u}" for u in params["mirror"]])

    if params["sticky"] is not None and params["sticky"]:
        args.append("--sticky")
    elif params["sticky"] is not None:
        args.append("--non-sticky")

    if params["enabled"] is not None and params["enabled"]:
        args.append("--enable")
    elif params["enabled"] is not None:
        args.append("--disable")

    rc, out, err = module.run_command(["pkg", "set-publisher"] + args + [name], check_rc=True)
    response = {
        "rc": rc,
        "results": [out],
        "msg": err,
        "changed": True,
    }
    if rc != 0:
        module.fail_json(**response)
    module.exit_json(**response)


def unset_publisher(module, publisher):
    if publisher not in get_publishers(module):
        module.exit_json()

    rc, out, err = module.run_command(["pkg", "unset-publisher", publisher], check_rc=True)
    response = {
        "rc": rc,
        "results": [out],
        "msg": err,
        "changed": True,
    }
    if rc != 0:
        module.fail_json(**response)
    module.exit_json(**response)


def get_publishers(module):
    rc, out, err = module.run_command(["pkg", "publisher", "-Ftsv"], True)

    lines = out.splitlines()
    keys = lines.pop(0).lower().split("\t")

    publishers = {}
    for line in lines:
        values = dict(zip(keys, map(unstringify, line.split("\t"))))
        name = values["publisher"]

        if name not in publishers:
            publishers[name] = {k: values[k] for k in ["sticky", "enabled"]}
            publishers[name]["origin"] = []
            publishers[name]["mirror"] = []

        if values["type"] is not None:
            publishers[name][values["type"]].append(values["uri"])

    return publishers


def unstringify(val):
    if val == "-" or val == "":
        return None
    elif val == "true":
        return True
    elif val == "false":
        return False
    else:
        return val


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2018, Sean Myers <sean.myers@redhat.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: rhsm_release
short_description: Set or Unset RHSM Release version
description:
  - Sets or unsets the release version used by RHSM repositories.
notes:
  - This module fails on an unregistered system. Use the M(community.general.redhat_subscription) module to register a system
    prior to setting the RHSM release.
  - It is possible to interact with C(subscription-manager) only as root, so root permissions are required to successfully
    run this module.
requirements:
  - Red Hat Enterprise Linux 6+ with subscription-manager installed
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  release:
    description:
      - RHSM release version to use.
      - To unset either pass V(null) for this option, or omit this option.
    type: str
author:
  - Sean Myers (@seandst)
"""

EXAMPLES = r"""
# Set release version to 7.1
- name: Set RHSM release version
  community.general.rhsm_release:
    release: "7.1"

# Set release version to 6Server
- name: Set RHSM release version
  community.general.rhsm_release:
    release: "6Server"

# Unset release version
- name: Unset RHSM release release
  community.general.rhsm_release:
    release:
"""

RETURN = r"""
current_release:
  description: The current RHSM release version value.
  returned: success
  type: str
"""

import os
import re

from ansible.module_utils.basic import AnsibleModule

# Matches release-like values such as 7.2, 5.10, 6Server, 8
# but rejects unlikely values, like 100Server, 1.100, 7server etc.
release_matcher = re.compile(r"\b\d{1,2}(?:\.\d{1,2}|Server|Client|Workstation|)\b")


def _sm_release(module, *args):
    # pass args to s-m release, e.g. _sm_release(module, '--set', '0.1') becomes
    # "subscription-manager release --set 0.1"
    sm_bin = module.get_bin_path("subscription-manager", required=True)
    cmd = [sm_bin, "release"] + list(args)
    # delegate nonzero rc handling to run_command
    return module.run_command(cmd, check_rc=True, expand_user_and_vars=False)


def get_release(module):
    # Get the current release version, or None if release unset
    rc, out, err = _sm_release(module, "--show")
    try:
        match = release_matcher.findall(out)[0]
    except IndexError:
        # 0'th index did not exist; no matches
        match = None

    return match


def set_release(module, release):
    # Set current release version, or unset if release is None
    if release is None:
        args = ("--unset",)
    else:
        args = ("--set", release)

    return _sm_release(module, *args)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            release=dict(type="str"),
        ),
        supports_check_mode=True,
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    if os.getuid() != 0:
        module.fail_json(msg="Interacting with subscription-manager requires root permissions ('become: true')")

    target_release = module.params["release"]

    # sanity check: the target release at least looks like a valid release
    if target_release and not release_matcher.findall(target_release):
        module.fail_json(msg=f'"{target_release}" does not appear to be a valid release.')

    # Will fail with useful error from s-m if system not subscribed
    current_release = get_release(module)

    changed = target_release != current_release
    if not module.check_mode and changed:
        set_release(module, target_release)
        # If setting the release fails, then a fail_json would have exited with
        # the s-m error, e.g. "No releases match '7.20'...".  If not, then the
        # current release is now set to the target release (job's done)
        current_release = target_release

    module.exit_json(current_release=current_release, changed=changed)


if __name__ == "__main__":
    main()
